@@ -3,7 +3,7 @@ use Mojo::Base 'Mojolicious::Plugin';
 
 # ABSTRACT: create form fields based on a definition in a JSON file
 
-our $VERSION = '1.02';
+our $VERSION = '1.03';
 
 use Carp;
 use File::Basename;
@@ -14,23 +14,20 @@ use List::Util qw(first);
 use Mojo::Asset::File;
 use Mojo::Collection;
 use Mojo::ByteStream;
-use Mojo::File;
+use Mojo::File qw(path);
 use Mojo::JSON qw(decode_json);
 
 use Mojolicious ();
 
 has dir => sub { ["."] };
 
-my $selected_value = Mojolicious->VERSION < 6.16 ? 'selected' : undef;
-my $checked_value  = Mojolicious->VERSION < 6.16 ? 'checked'  : undef;
-
 sub register {
     my ( $self, $app, $config ) = @_;
 
     $config //= {};
 
     if ( $config->{template_file} ) {
-        $config->{template} = Mojo::File->new( $app->home, 'templates', $config->{template_file} )->slurp;
+        $config->{template} = path( $app->home, 'templates', $config->{template_file} )->slurp;
         $config->{template} //= $app->renderer->get_data_template( $config->{template_file} );
     }
 
@@ -64,7 +61,7 @@ sub register {
             for my $dir ( @{ $self->dir } ) {
                 my $dir = IO::Dir->new($dir);
 
-              FILE:
+                FILE:
                 while ( my $file = $dir->read ) {
                     next FILE if $file !~ m{\.json\z};
 
@@ -125,12 +122,19 @@ sub register {
             my $params_hash = $c->req->params->to_hash;
             my @param_names = keys %{ $params_hash || {} };
 
-            my %params = map { $_ => $c->every_param($_) } @param_names;
+            my %params;
+            for my $name ( @param_names ) {
+                my $param_values = $c->every_param($name);
+
+                $params{$name} = @{$param_values || [] } > 1 ?
+                    $param_values : $param_values->[-1];
+            }
+
             $validation->input( \%params );
 
             my %errors;
 
-          FIELD:
+            FIELD:
             for my $field ( @{$config} ) {
                 if ( 'HASH' ne ref $field ) {
                     $app->log->error('Field definition must be a HASH - skipping field');
@@ -146,11 +150,13 @@ sub register {
                     next FIELD;
                 }
 
+                my $filters = $field->{validation}->{filters};
+
                 my $name         = $field->{name} // $field->{label} // '';
                 my $global_error = 1;
 
                 if ( $field->{validation}->{required} ) {
-                    $validation->required($name);
+                    $validation->required($name, @{ $filters || [] });
 
                     my $value = $field->{validation}->{required};
                     if ( ref $value && 'HASH' eq ref $value ) {
@@ -158,14 +164,15 @@ sub register {
                     }
                 }
                 else {
-                    $validation->optional($name);
+                    $validation->optional($name, @{ $filters || [] });
                 }
 
-              RULE:
+                RULE:
                 for my $rule ( sort keys %{ $field->{validation} } ) {
                     last RULE if !defined $params{$name};
 
                     next RULE if $rule eq 'required';
+                    next RULE if $rule eq 'filters';
 
                     my $value  = $field->{validation}->{$rule};
                     my $ref    = ref $value;
@@ -220,7 +227,7 @@ sub register {
 
             my %fields_to_show = map { $_ => 1 } @{ $params{fields} || [] };
 
-          FIELD:
+            FIELD:
             for my $field ( @{$field_config} ) {
                 next FIELD if %fields_to_show && !$fields_to_show{ $field->{name} };
 
@@ -346,8 +353,9 @@ sub _build_form_field {
 
     $form_field = Mojo::ByteStream->new($form_field);
 
-    my $template = $field->{template} // $plugin_config->{templates}->{$orig_type}
-      // $plugin_config->{template};
+    my $template = $field->{template}
+        // $plugin_config->{templates}->{$orig_type}
+        // $plugin_config->{template};
 
     if ( $template && $type ne 'hidden' ) {
         my $label = $field->{label} // '';
@@ -540,7 +548,7 @@ sub _transform_hash_values {
     my $counter = 0;
     my %mapping;
 
-  KEY:
+    KEY:
     for my $key ( keys %{$data} ) {
         if ( ref $data->{$key} ) {
             my @group_values = $self->_get_select_values( $c, +{ data => $data->{$key} }, %params );
@@ -550,8 +558,8 @@ sub _transform_hash_values {
         else {
             my %opts;
 
-            $opts{disabled} = 'disabled'      if $params{disabled}->{$key};
-            $opts{selected} = $selected_value if $params{selected}->{$key};
+            $opts{disabled} = 'disabled' if $params{disabled}->{$key};
+            $opts{selected} = undef      if $params{selected}->{$key};
 
             #$opts{selected} = undef if $params{selected}->{$key};
 
@@ -591,18 +599,17 @@ sub _transform_array_values {
 
         my %opts;
 
-        $opts{disabled} = 'disabled'      if $params{disabled}->{$value};
-        $opts{selected} = $selected_value if $params{selected}->{$value};
+        $opts{disabled} = 'disabled' if $params{disabled}->{$value};
+        $opts{selected} = undef      if $params{selected}->{$value};
 
         #$opts{selected} = undef if $params{selected}->{$value};
 
         push @values, [ $value => $value, %opts ];
     }
 
-    @values =
-      $numeric
-      ? sort { $a->[0] <=> $b->[0] } @values
-      : sort { $a->[0] cmp $b->[0] } @values;
+    @values = $numeric
+        ? sort { $a->[0] <=> $b->[0] } @values
+        : sort { $a->[0] cmp $b->[0] } @values;
 
     return @values;
 }
@@ -619,20 +626,23 @@ sub _radio {
 
     my $field_params = $params{$name} || {},
 
-      my %select_params = (
+    my %select_params = (
         disabled => $self->_get_highlighted_values( $field, 'disabled' ),
         selected => $self->_get_highlighted_values( $field, 'selected' ),
-      );
+    );
 
     my $stash_values = $c->every_param($name);
     if ( scalar( @{ $stash_values || [] } ) == 0 && defined( $c->stash($name) ) ) {
         my $local_stash = $c->stash($name);
         $stash_values = ref $local_stash ? $local_stash : [$local_stash];
     }
+
     my $reset;
     if ( @{ $stash_values || [] } ) {
-        $select_params{selected} =
-          $self->_get_highlighted_values( +{ selected => $stash_values }, 'selected', );
+        $select_params{selected} = $self->_get_highlighted_values(
+            +{ selected => $stash_values },
+            'selected',
+        );
         $reset = 1;
     }
 
@@ -659,7 +669,7 @@ sub _radio {
         }
 
         if ( $select_params{selected}->{$radio_value} ) {
-            $value_attributes{checked} = $checked_value;
+            $value_attributes{checked} = undef;
         }
 
         my $local_label = '';
@@ -707,16 +717,17 @@ sub _checkbox {
 
     my $field_params = $params{$name} || {},
 
-      my %select_params = (
+    my %select_params = (
         disabled => $self->_get_highlighted_values( $field, 'disabled' ),
         selected => $self->_get_highlighted_values( $field, 'selected' ),
-      );
+    );
 
     my $stash_values = $c->every_param($name);
     if ( scalar( @{ $stash_values || [] } ) == 0 && defined( $c->stash($name) ) ) {
         my $local_stash = $c->stash($name);
         $stash_values = ref $local_stash ? $local_stash : [$local_stash];
     }
+
     my $reset;
     if ( @{ $stash_values || [] } ) {
         $select_params{selected} =
@@ -748,7 +759,7 @@ sub _checkbox {
         }
 
         if ( $select_params{selected}->{$checkbox_value} ) {
-            $value_attributes{checked} = $checked_value;
+            $value_attributes{checked} = undef;
         }
 
         my $local_label = '';
@@ -820,7 +831,7 @@ Mojolicious::Plugin::FormFieldsFromJSON - create form fields based on a definiti
 
 =head1 VERSION
 
-version 1.02
+version 1.03
 
 =head1 SYNOPSIS
 
@@ -2074,6 +2085,13 @@ you can set a scalar:
 
 Validation checks are done in asciibetical order.
 
+You can also use the L<filters|Mojolicious::Validator/FILTERS>:
+
+  "validation" : {
+      "size" : [ 2, 5 ],
+      "filters" : [ "trim" ]
+  },
+
 =head2 Check a string for its length
 
 This is a simple check for the length of a string
